
# Copyright (c) 2018 Josef Wachtler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
This module provides some helper functions for the LTI Provider.
"""

from django.core.urlresolvers import reverse
from django.core.exceptions import ImproperlyConfigured


def get_by_py_path(py_path):
    """
    Imports and returns a python callable.

    Keyword arguments:
        py_path -- callable to load
    """
    parts = py_path.split('.')
    module = ".".join(parts[:-1])
    m = __import__(module)
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m


def reverse_from_settings(parameter_to_view):
    """
    takes a tuple where the first element is the name of a view and the
    optional second parameter is a tuple, a list or a dictonary of the
    parameters for the view.

    keyword arguments:
        - parameter_to_view -- tuple of parameters
    """
    l = len(parameter_to_view)
    if l == 1:
        return reverse(parameter_to_view[0])
    elif l == 2:
        return reverse(parameter_to_view[0], args=parameter_to_view[1])
    else:
        raise ImproperlyConfigured('too much parameters')




# Copyright (c) 2018 Josef Wachtler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
This module provides backends for the lti_provider.
"""

from hashlib import sha1

from django.core.exceptions import PermissionDenied
from django.conf import settings
from django.contrib.auth.models import User

from lti_provider.utils import get_by_py_path
from lti_provider.validators import LTIValidator


class LTIAuthBackend(object):
    """
    This is a authentication backend for LTI.
    """

    def authenticate(self, request, tool_provider=None):
        """
        Authenticates a user via a LTI request from a consumer.

        Keyword arguments:
            - request -- the HttpRequest
            - tool_provider -- the LTI tool provider instance
        """
        if not tool_provider:
            return None

        validator = LTIValidator()
        ok = tool_provider.is_valid_request(validator)
        if not ok:
            raise PermissionDenied

        try:
            uid = tool_provider.launch_params['user_id']
        except KeyError:
            raise PermissionDenied
        try:
            email = tool_provider.launch_params[
                'lis_person_contact_email_primary']
        except KeyError:
            raise PermissionDenied
        try:
            given_name = tool_provider.launch_params['lis_person_name_given']
        except KeyError:
            given_name = 'LTI'
        try:
            family_name = tool_provider.launch_params['lis_person_name_family']
        except KeyError:
            family_name = 'LTI'

        username = email + '_' + sha1(uid.encode('utf-8')).hexdigest()
        username = username[:120]
        user = None
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            user = User(username=username, password='LTI user')
            user.set_unusable_password()
            user.email = email
            user.first_name = given_name[:30]
            user.last_name = family_name[:30]
            user.save()
            try:
                hook = get_by_py_path(
                    settings.LTI_PROVIDER['HOOK_AFTER_USER_CREATION'])
                hook(user)
            except KeyError:
                pass

        if not user:
            raise PermissionDenied

        return user

    def get_user(self, user_id):
        """
        Returns a user object.

        Keyword arguments:
            - user_id -- the id of the user to return
        """
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-09-14 14:03
from __future__ import unicode_literals

from django.db import migrations, models
import lti_provider.field_validators


class Migration(migrations.Migration):

    dependencies = [
        ('lti_provider', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='consumer',
            name='key',
            field=models.CharField(max_length=128, unique=True, validators=[lti_provider.field_validators.validate_oauth_chars, lti_provider.field_validators.validate_oauth_length], verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='consumer',
            name='secret',
            field=models.CharField(max_length=128, unique=True, validators=[lti_provider.field_validators.validate_oauth_chars, lti_provider.field_validators.validate_oauth_length], verbose_name='Secret'),
        ),
    ]




# Copyright (c) 2018 Josef Wachtler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
This module provides validators for the LTI request of the lti_provider-App.
"""

import logging
from oauthlib.oauth1 import RequestValidator
from lti_provider.models import Consumer, TimestampAndNonce

# Get an instance of a logger
logger = logging.getLogger('LTI.lti_provider')


class LTIValidator(RequestValidator):
    """
    This validator implements the RequestValidator from the oauthlib.
    It implements only the methods required for a LTI request.
    """

    @property
    def enforce_ssl(self):
        """
        Returns false to not check for ssl.
        """
        logger.debug('called')
        return False

    @property
    def nonce_length(self):
        """
        Returns the min (5) and the max (50) length of a nonce.
        """
        return 5, 50

    @property
    def dummy_client(self):
        """
        Returns a dummy key used for false consumers.
        """
        logger.debug('called')
        return "dummy_client_key_123456"

    def get_client_secret(self, client_key, request):
        """
        Returns the secret of a consumer identified by its key.

        Keyword arguments:
            - client_key -- the key of the consumer
            - request -- calling request
        """
        logger.debug('called')
        try:
            c = Consumer.objects.get(key=client_key)
            return str(c.secret)
        except Consumer.DoesNotExist:
            return 'dummy_client_sec_123456'

    def validate_client_key(self, client_key, request):
        """
        Returns true if the given key exists.

        Keyword arguments:
            - client_key -- the key of the consumer
            - request -- calling request
        """
        logger.debug('called')
        try:
            n = Consumer.objects.filter(key=client_key).count()
            if n > 0:
                return True
            else:
                return False
        except Consumer.DoesNotExist:
            return False

    def validate_timestamp_and_nonce(self, client_key, timestamp, nonce,
                                     request, request_token=None,
                                     access_token=None):
        """
        Returns true if the given timestamp and nonce are valid for the
        consumer.

        Keyword arguments:
            - client_key -- the key of the consumer
            - timestamp -- the timestamp to validate
            - nonce -- the nonce to validate
            - request -- calling request
            - request_token -- unused for LTI
            - access_token -- unused for LTI
        """
        logger.debug('called')
        tn = TimestampAndNonce.objects.filter(
            consumer__key=client_key,
            timestamp=timestamp,
            nonce=nonce).count()
        if tn > 0:
            return False
        else:
            try:
                c = Consumer.objects.get(key=client_key)
            except Consumer.DoesNotExist:
                logger.debug('wrong consumer key')
                return False
            t = TimestampAndNonce(
                consumer=c,
                timestamp=timestamp,
                nonce=nonce)
            t.save()
            return True




# Copyright (c) 2018 Josef Wachtler
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
This module provides the views of the lti_provider.
"""


import logging
from lti import ToolConfig
from lti.contrib.django import DjangoToolProvider
from django.http import HttpResponse, HttpResponseBadRequest, \
    HttpResponseRedirect
from django.urls import reverse
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth import authenticate, login, logout
from django.conf import settings

from lti_provider.utils import reverse_from_settings

# Get an instance of a logger
logger = logging.getLogger('LTI.lti_provider')


@csrf_exempt
def tool_config(request):
    """
    This view returns a xml file with information about the LTI provider.

    Decorators:
        - csrf_exempt -- disable csrf protection

    Keyword arguments:
        - request -- calling HttpRequest
    """
    app_title = settings.LTI_PROVIDER['TITLE']
    app_description = settings.LTI_PROVIDER['DESCRIPTION']
    launch_url = request.build_absolute_uri(
        reverse('lti_provider.views.lti_launch'))

    lti_tool_config = ToolConfig(
        title=app_title,
        launch_url=launch_url,
        secure_launch_url=launch_url,
        description=app_description
    )

    return HttpResponse(lti_tool_config.to_xml(), content_type='text/xml')


@csrf_exempt
def lti_launch(request):
    """
    This is the entry point via LTI. It authenticates the user and redirects
    to the requested destination.

    Decorators:
        - csrf_exempt -- disable csrf protection

    Keyword arguments:
        - request -- calling HttpRequest
    """
    if request.user.is_authenticated():
        logout(request)

    try:
        tool_provider = DjangoToolProvider.from_django_request(request=request)
    except:
        return HttpResponseBadRequest('wrong config')

    user = authenticate(request=request, tool_provider=tool_provider)
    failed = settings.LTI_PROVIDER['FAILED_VIEW']
    if user is not None:
        if user.is_active:
            login(request, user)
            ps = settings.LTI_PROVIDER['PARAMETERS_TO_VIEW']
            for p in ps:
                ok = True
                ap = {}
                for pn in p[0]:
                    gp = tool_provider.get_custom_param(pn)
                    ap[pn] = gp
                    if not gp:
                        ok = False
                if ok:
                    try:
                        return HttpResponseRedirect(reverse(
                            p[1], kwargs=ap))
                    except:
                        pass
            default = settings.LTI_PROVIDER['DEFAULT_VIEW']
            return HttpResponseRedirect(reverse_from_settings(default))
        else:
            return HttpResponseRedirect(reverse_from_settings(failed))
    else:
        return HttpResponseRedirect(reverse_from_settings(failed))
